using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Tao.OpenGl;
using Tao.Platform.Windows;
using PendulumEngine;
using System.Threading;
using System.Drawing.Imaging;
using System.IO;




namespace Pendulum
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Pendulum : System.Windows.Forms.Form
	{
		#region Variabler
		private static IntPtr hDC;                                              // GDI Device Context
		private static IntPtr hRC;                                              // Rendering Context
		private System.Windows.Forms.Timer RefreshTimer;						// Timer som benyttes til  f en konstant refresh rate
		private System.ComponentModel.IContainer components;					// Container for komponenter
		PendulumModel PM = new PendulumModel();									// Data for tegning av Pendel, importert fra 3DS Max
		static SharedEngineData ED = new SharedEngineData();					// Data som deles mellom simulator og Opengl Applikasjon
		static Engine InvEngPendulum;											// Matematisk simulator for pendel bevegelser
		private static int[] texture = new int[1];                              // Storage For One Texture ( NEW )
		private System.Windows.Forms.Label label3;								// Statisk tekst
		private System.Windows.Forms.Label label2;								// Statisk tekst
		private System.Windows.Forms.TrackBar trackBarZoom;						// Trackbar som lar bruker utfre zoom
		private System.Windows.Forms.Panel panel1;								// Panel som opengl applikasjon bli rendret til
		private System.Windows.Forms.Panel panel2;								// Brukes til  lage skillilinje i panel1
		private System.Windows.Forms.Panel panel3;								// Brukes til  lage skillilinje i panel1
		private System.Windows.Forms.Panel panel5;								// Brukes til  lage skillilinje i panel1
		private System.Windows.Forms.TrackBar trackBarRoatatePlatform;			// Scrollbar som lar bruker rotere plattform
		private System.Windows.Forms.Timer CpuUsagetimer;						// Timer som benyttes til  beregne total cpu bruk
		private System.Windows.Forms.ProgressBar progressBarCpuUsage;			// Progressbar som benyttes til  presentere cpu forbruk
		private System.Windows.Forms.Label label4;								// Statisk tekst
		private PerformanceCounter cpuCounter;									// Counter som benyttes til  beregne cpu forbruk
		private static int fontbase;                                            // Base Display List For The Font Set
		private static Gdi.GLYPHMETRICSFLOAT[] gmf = new Gdi.GLYPHMETRICSFLOAT[256];	// Lagringsplass for fonter
		private System.Windows.Forms.Label label5;								// Statisk tekst
		private System.Windows.Forms.Label label6;								// Statisk tekst
		private System.Windows.Forms.Label label7;								// Statisk tekst
		private System.Windows.Forms.Label label8;								// Statisk tekst
		private System.Windows.Forms.GroupBox groupBox1;						// Gruppeboks for diverse styre muligheter
		private System.Windows.Forms.CheckBox checkBoxView3;					// Checkboxs som slr vindu 3 av/p
		private System.Windows.Forms.CheckBox checkBoxView4;					// Checkboxs som slr vindu 4 av/p
		private System.Windows.Forms.CheckBox checkBoxView2;					// Checkboxs som slr vindu 2 av/p
		private System.Windows.Forms.CheckBox checkBoxView1;					// Checkboxs som slr vindu 1 av/p
		private System.Windows.Forms.CheckBox checkBoxTexture;					// Checkboxs som slr teksturen p plattform av p
		private System.Windows.Forms.CheckBox checkBox2;						// Checkboxs som slr lys av/p
		private System.Windows.Forms.GroupBox groupBox2;						// Gruppeboks for data som sendes til simulator
		private System.Windows.Forms.CheckBox checkBoxRegulationOn;				// Checkboxs som slr regulator p/av
		private System.Windows.Forms.Label label1;								// Statisk tekst
		private System.Windows.Forms.TrackBar trackBarReference;				// Trackbar som setter nsket pososjon for regulator
		private System.Windows.Forms.CheckBox checkBoxNoiseOn;					// Checkboxs som slr p/av sty til regulatoren
		private float Angle = 0;
		#endregion 

		#region Konstruktr
		/// <summary>
		/// Standard konstruktr
		/// </summary>
		public Pendulum()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			this.SetStyle(ControlStyles.DoubleBuffer, true);                    // Buffer Control

			this.Resize += new EventHandler(Form_Resize);						// On Resize Event Call Form_Resize

			// Lager opengl vinduet
			if(!CreateGLWindow("Grafisk Databehandling, Vren 2005, Rune Wilhelm Dragsnes"))
			{
				return;                                                         // Avslutt vis ikke vindu ble laget
			}

			RefreshTimer.Enabled = true;
			InvEngPendulum = new Engine(ED);
			Thread InvEngPendulumThread = new Thread(new ThreadStart(InvEngPendulum.StartEngine));
			InvEngPendulumThread.Priority = ThreadPriority.Normal;
			InvEngPendulumThread.Start();
			cpuCounter = new PerformanceCounter();
			cpuCounter.CategoryName = "Processor"; 
			cpuCounter.CounterName = "% Processor Time"; 
			cpuCounter.InstanceName = "_Total"; 
		}

		#endregion

		#region Lager en Rendering Context
		/// <summary>
		/// Funskjon som setter opp en vindu for opengl applikajsonen
		/// </summary>
		/// <param name="title"></param>
		/// <returns></returns>
		private bool CreateGLWindow(string title) 
		{
			int pixelFormat;                                                    // Holds The Results After Searching For A Match
			int bits = 16;														//	Opplsning
			GC.Collect();                                                       //	Request A Collection
			// This Forces A Swap
			Kernel.SetProcessWorkingSetSize(Process.GetCurrentProcess().Handle, -1, -1);


			
			Cursor.Show();														// Show Mouse Pointer

			this.Text = title;                                                  // Set Window Title

			Gdi.PIXELFORMATDESCRIPTOR pfd = new Gdi.PIXELFORMATDESCRIPTOR();    // pfd forteller windows vordan vi nsker ting skal bli
			pfd.nSize = (short) Marshal.SizeOf(pfd);                            // strrelsen p 'Pixel Format Descriptor'
			pfd.nVersion = 1;                                                   // Versjonsnummer
			pfd.dwFlags = Gdi.PFD_DRAW_TO_WINDOW |                              // Formatet m sttte tegning til vindu
				Gdi.PFD_SUPPORT_OPENGL |                                        // Formatet m sttte opengl
				Gdi.PFD_DOUBLEBUFFER;                                           // Formatet m sttte dobbel buffer
			pfd.iPixelType = (byte) Gdi.PFD_TYPE_RGBA;                          // Foresprsel om RGBA Format
			pfd.cColorBits = (byte) bits;                                       // Farge dybde
			pfd.cRedBits = 0;                                                   // Color Bits Ignored
			pfd.cRedShift = 0;
			pfd.cGreenBits = 0;
			pfd.cGreenShift = 0;
			pfd.cBlueBits = 0;
			pfd.cBlueShift = 0;
			pfd.cAlphaBits = 0;                                                 // Ingen alpha buffer
			pfd.cAlphaShift = 0;                                                // Shift Bit ignoreres
			pfd.cAccumBits = 0;                                                 
			pfd.cAccumRedBits = 0;                                              
			pfd.cAccumGreenBits = 0;
			pfd.cAccumBlueBits = 0;
			pfd.cAccumAlphaBits = 0;
			pfd.cDepthBits = 16;                                                // 16Bit Z-Buffer (Dybde buffer)
			pfd.cStencilBits = 0;                                               
			pfd.cAuxBuffers = 0;                                                
			pfd.iLayerType = (byte) Gdi.PFD_MAIN_PLANE;                         
			pfd.bReserved = 0;                                                  // reservert
			pfd.dwLayerMask = 0;                                                
			pfd.dwVisibleMask = 0;
			pfd.dwDamageMask = 0;

			hDC = User.GetDC(panel1.Handle);                                    // henter Device Context

			if(hDC == IntPtr.Zero)												// fikk vi en Device Context?
			{                                            
				KillGLWindow();                                                 
				MessageBox.Show("Ka ikke lage en Device Context.", "ERROR",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}

			pixelFormat = Gdi.ChoosePixelFormat(hDC, ref pfd);                  // Finner beste Pixel Format
			if(pixelFormat == 0)												// Fant vi et Pixel Format?
			{                                              
				KillGLWindow();                                                 
				MessageBox.Show("Kan ikke finne et egnet Piksel Format.", "ERROR",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}

			if(!Gdi.SetPixelFormat(hDC, pixelFormat, ref pfd))					// Kan vi sette piksel format 
			{             
				KillGLWindow();                                                 
				MessageBox.Show("Kan ikke sette piksel format.", "ERROR",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}

			hRC = Wgl.wglCreateContext(hDC);                                    // Henter Rendering Context
			if(hRC == IntPtr.Zero)												// Fikk vi en RC?
			{                                            
				KillGLWindow();                                                 
				MessageBox.Show("Kan ikke lage en GL Rendering Context.", "ERROR",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}

			if(!Wgl.wglMakeCurrent(hDC, hRC))									// Prver  aktivere RC
			{                                 
				KillGLWindow();               
				MessageBox.Show("Kan ikke aktivisere GL Rendering Context.", "ERROR",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}

			if(!InitGL())														// Initsierer OpenGL vinduet
			{                                                     
				KillGLWindow();                                                 
				MessageBox.Show("Kan ikke initsiere opengl vinduet.", "ERROR",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}
			return true;                                                        
		}
		#endregion

		#region Sletter rendering context
		/// <summary>
		/// Funksjon som sletter opengl vinduet
		/// </summary>
		private void KillGLWindow() 
		{
			if(hRC != IntPtr.Zero)		// Har vi en RC ?
			{                                            
				if(!Wgl.wglMakeCurrent(IntPtr.Zero, IntPtr.Zero)) 
				{            
					MessageBox.Show("Frigivelse av DC og RC er mislykket.", "SHUTDOWN ERROR",
						MessageBoxButtons.OK, MessageBoxIcon.Error);
				}

				if(!Wgl.wglDeleteContext(hRC)) 
				{                                
					MessageBox.Show("Frigivelse av RC er mislykket", "SHUTDOWN ERROR",
						MessageBoxButtons.OK, MessageBoxIcon.Error);
				}

				hRC = IntPtr.Zero;                                              // Setter RC til null
			}

			if(hDC != IntPtr.Zero)		// Har vi en DC?
			{                                           
				if(this != null && !this.IsDisposed) // har vi et vindu?
				{                          
					if(this.Handle != IntPtr.Zero)	// Har vi en vindu HANDLE ?
					{                            
						if(!User.ReleaseDC(this.Handle, hDC))	// Klarer vi  frigi DC?
						{                 
							MessageBox.Show("Sletting av DC var misslykket.", "SHUTDOWN ERROR",
								MessageBoxButtons.OK, MessageBoxIcon.Error);
						}
					}
				}

				hDC = IntPtr.Zero;                                              // Setter DC til null
			}

			if(this != null)	// har vi en form?
			{                                                  
				this.Hide();    // gjm vinduet
				this.Close();   // avslutt
			}
			KillFont();
		}

		#endregion
	
		#region Init openGL
		/// <summary>
		/// Funskjon som initsierer opengl applikasjonen
		/// </summary>
		/// <returns></returns>
		private static bool InitGL() 
		{
			if(!LoadGLTextures())	//Laster inn teksturen
			{                                             
				return false;       
			}
			Gl.glEnable(Gl.GL_TEXTURE_2D);                                      // Enable Texture Mapping
			Gl.glShadeModel(Gl.GL_SMOOTH);                                      // Enable Smooth Shading
			Gl.glClearColor(0, 0, 0, 0.5f);                                     // Svart bakgrunn
			Gl.glClearDepth(1);                                                 // Dybde buffer konfig
			Gl.glEnable(Gl.GL_DEPTH_TEST);                                      // Slr p dybde testing
			Gl.glDepthFunc(Gl.GL_LEQUAL);                                       // Type dybde testing
			Gl.glHint(Gl.GL_PERSPECTIVE_CORRECTION_HINT, Gl.GL_NICEST);         // setter fine perspektiv kalkulasjoner

			Gl.glEnable     ( Gl.GL_CULL_FACE  );
			Gl.glCullFace   ( Gl.GL_BACK       );
			Gl.glFrontFace  ( Gl.GL_CCW );

			float []ambient = {0.8f,0.8f,0.8f,1.0f };
			float []diffuse = {1.0f,1.0f,1.0f,1.0f };
			float []position = {20.0f,30.0f,10.0f,0.0f };
			float []lmodel_ambient = {0.4f,0.4f,0.4f,1.0f };

			Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, ambient);
			Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, diffuse);
			Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, position);
	
			Gl.glEnable(Gl.GL_LIGHT0);
			Gl.glEnable(Gl.GL_LIGHTING);
			BuildFont();                                                        // Lager fontene
			return true;
		}

		#endregion

		#region Resize openGL
		/// <summary>
		/// Event handler som bli kalt nr formen endrer strrelse
		/// </summary>
		/// <param name="width"></param>
		/// <param name="height"></param>
		private static void ReSizeGLScene(int width, int height) 
		{
			if(height == 0) 
			{																	// Forhindrer Exception (Divide By Zero...)
				height = 1;                                                     // ved  sette hyde lik 1
			}

			Gl.glViewport(0, 0, width, height);                                 // Restter  Viewport
			Gl.glMatrixMode(Gl.GL_PROJECTION);                                  // Velger Projection Matrix
			Gl.glLoadIdentity();                                                // Restter Projection Matrix
			Glu.gluPerspective(45, width / (double) height, 0.1, 2000);         // Beregner forholdet forvinduet
			Gl.glMatrixMode(Gl.GL_MODELVIEW);                                   // Velger Modelview Matrix
			Gl.glLoadIdentity();                                                // Restter Modelview Matrix
		}

		#endregion

		#region Tegne funksjoner

		/// <summary>
		/// Generell tegne rutine som tegner objekter lagret i tabeller
		/// </summary>
		/// <param name="Normal"></param>
		/// <param name="Indicies"></param>
		/// <param name="Vertices"></param>
		private void DrawObject(float [] Normal, int [] Indicies, float [] Vertices )
		{
			int count = Indicies.Length/9;
			for (int i=0;i<count;i++)
			{
				for (int ind=0; ind<3; ind++)
				{
					int indv = Indicies[i*9+ind];
					int indn = Indicies[i*9+3+ind];

					Gl.glNormal3f(Normal[indn*3+0],Normal[indn*3+1],Normal[indn*3+2]);
					Gl.glVertex3f(Vertices[indv*3+0],Vertices[indv*3+1],Vertices[indv*3+2]);
				}
			}
		}

		/// <summary>
		/// Funksjon som tegner hjulene til pendelen, dette gjres da disse skal roteres uavhengig av 
		/// resten av pendelen
		/// </summary>
		private void DrawWheels()
		{
			Gl.glBegin(Gl.GL_TRIANGLES);
			SetMaterial(MaterialType.ROD);
			DrawObject(PM.LWNaveNormals, PM.LWNaveIndicies, PM.LWNaveVertices);
			DrawObject(PM.LeftWheelNormals, PM.LeftWheelIndicies, PM.LeftWheelVertices);
			SetMaterial(MaterialType.VIT);
			DrawObject(PM.LWSpoke1Normals, PM.LWSpoke1Indicies, PM.LWSpoke1Vertices);
			DrawObject(PM.LWSpoke2Normals, PM.LWSpoke2Indicies, PM.LWSpoke2Vertices);
			DrawObject(PM.LWSpoke3Normals, PM.LWSpoke3Indicies, PM.LWSpoke3Vertices);
			DrawObject(PM.LWSpoke4Normals, PM.LWSpoke4Indicies, PM.LWSpoke4Vertices);
			DrawObject(PM.LWSpoke5Normals, PM.LWSpoke5Indicies, PM.LWSpoke5Vertices);
			DrawObject(PM.LWSpoke6Normals, PM.LWSpoke6Indicies, PM.LWSpoke6Vertices);
			DrawObject(PM.LWSpoke7Normals, PM.LWSpoke7Indicies, PM.LWSpoke7Vertices);
			DrawObject(PM.LWSpoke8Normals, PM.LWSpoke8Indicies, PM.LWSpoke8Vertices);

			SetMaterial(MaterialType.ROD);
			DrawObject(PM.RWNaveNormals, PM.RWNaveIndicies, PM.RWNaveVertices);
			DrawObject(PM.RightWheelNormals, PM.RightWheelIndicies, PM.RightWheelVertices);
			SetMaterial(MaterialType.VIT);
			DrawObject(PM.RWSpoke1Normals, PM.RWSpoke1Indicies, PM.RWSpoke1Vertices);
			DrawObject(PM.RWSpoke2Normals, PM.RWSpoke2Indicies, PM.RWSpoke2Vertices);
			DrawObject(PM.RWSpoke3Normals, PM.RWSpoke3Indicies, PM.RWSpoke3Vertices);
			DrawObject(PM.RWSpoke4Normals, PM.RWSpoke4Indicies, PM.RWSpoke4Vertices);
			DrawObject(PM.RWSpoke5Normals, PM.RWSpoke5Indicies, PM.RWSpoke5Vertices);
			DrawObject(PM.RWSpoke6Normals, PM.RWSpoke6Indicies, PM.RWSpoke6Vertices);
			DrawObject(PM.RWSpoke7Normals, PM.RWSpoke7Indicies, PM.RWSpoke7Vertices);
			DrawObject(PM.RWSpoke8Normals, PM.RWSpoke8Indicies, PM.RWSpoke8Vertices);
			Gl.glEnd();	
		}

		/// <summary>
		/// Funksjon som tegner pendelen med unntak av brakettene og hjulene.
		/// </summary>
		private void DrawBoddy()
		{
			Gl.glBegin(Gl.GL_TRIANGLES);
			DrawObject(PM.EngineAxleNormals, PM.EngineAxleIndicies, PM.EngineAxleVertices );
			DrawObject(PM.SmalGearNormals, PM.SmalGearIndices, PM.SmalGearVertices);
			DrawObject(PM.LargeGearNotmals, PM.LargeGearIndicies, PM.LargeGearVertices);
			DrawObject(PM.AxleNormals, PM.AxleIndicies, PM.AxleVertices);
			SetMaterial(MaterialType.KOBBER);
			DrawObject(PM.BaseNormals, PM.BaseFaceIndicies, PM.BaseVertex );
			SetMaterial(MaterialType.GRAA);
			DrawObject(PM.EngineNormals, PM.EngineIndicies, PM.EngineVertices );
			SetMaterial(MaterialType.GRONN);
			DrawObject(PM.MotherBoardNormals, PM.MotherBoardIndicies, PM.MotherBoardVertices);
			Gl.glEnd();
		}

		/// <summary>
		/// Funksjon som tegner alle braketter, disse tegnes selvstendig da de skal gjres transparange
		/// </summary>
		private void DrawBrackets()
		{
			Gl.glBegin(Gl.GL_TRIANGLES);
			SetMaterial(MaterialType.GRAA);
			DrawObject(PM.RightWheelBracketNormals, PM.RightWheelBracketIndicies, PM.RightWheelBracketVertices);
			DrawObject(PM.LeftWheelBracketNormals, PM.LeftWheelBracketIndicies, PM.LeftWheelBracketVertices);
			DrawObject(PM.LeftEngineBracketNormals, PM.LeftEngineBracketIndicies, PM.LeftEngineBracketVertices );
			DrawObject(PM.LeftCardBracketNormals, PM.LeftCardBracketIndicies, PM.LeftCardBracketVertices);
			DrawObject(PM.RightCardBracketNormals, PM.RightCardBracketIndicies, PM.RightCardBracketVertices );
			DrawObject(PM.RightEngineNormals, PM.RightEngineIdicies, PM.RightEngineVertices );
			Gl.glEnd();
		}

		/// <summary>
		/// Funksjon som tegner plattformen pendelen kjrer p, det blir her ogs lagt til en tekstur
		/// som er lastet inn tidligere fra en fil.
		/// </summary>
		private void DrawDisk()
		{
			Glu.GLUquadric glpQ;
			glpQ = Glu.gluNewQuadric();
			if (checkBoxTexture.Checked)
				Glu.gluQuadricTexture(glpQ,1);
			Glu.gluDisk(glpQ,0f,300,80,1);
			Gl.glTranslatef(0,0,-10);
			Glu.gluCylinder(glpQ, 300, 300, 10,100,30);
			Glu.gluDeleteQuadric(glpQ);
		}

		/// <summary>
		/// Tegner en markr som indikerer punktet som Simulatoren skal regulere seg inn til
		/// </summary>
		private void DrawMarker()
		{
			Glu.GLUquadric glpQ;
			glpQ = Glu.gluNewQuadric();
			Glu.gluSphere(glpQ, 10, 10, 10);// gluDisk(glpQ,0f,10,10,1);
			Glu.gluDeleteQuadric(glpQ);
		}

		/// <summary>
		/// Funksjon som tegner hele den inverterte pendelen, her tar man ogs hyde for rotasjon av
		/// hjulene i henhold til forflyttning langs x-aksen. 
		/// </summary>
		public void DrawPendel()
		{
			DrawBoddy();
			Gl.glPushMatrix();
			Gl.glRotatef((((float)ED.X1_K*200f)/(2.0f*(float)Math.PI*20.0f))*360f,0f,1f,0f);
			Gl.glTranslatef(0,0,2f);
			DrawWheels();
			Gl.glPopMatrix();
			Gl.glBlendFunc(Gl.GL_ONE,Gl.GL_ONE);
			Gl.glEnable(Gl.GL_BLEND);
			DrawBrackets();
			Gl.glDisable(Gl.GL_BLEND);
		}
		
		/// <summary>
		/// Hovedfunksjon for tegning av pendel, denne funksjonen stter opp rotasjon av plattform
		/// og posisjon av pendel. Videre kalles funksjoner for tegning av marker, plattform og pendel.
		/// </summary>
		public void DrawPendulumScene()
		{
			// Modellen er modulert etter flgende akser
			//
			//  |----|     Z
			//  |    |     |
			//  |    |     |
			//  |    |     /-----Y
			//  ------    /
			//  0    0   X
			
			// Culling m settesda tegning av tekst bruker display lister som endrer disse
			Gl.glEnable     ( Gl.GL_CULL_FACE  );
			Gl.glCullFace   ( Gl.GL_BACK       );
			Gl.glFrontFace  ( Gl.GL_CCW );

			Gl.glRotatef((trackBarRoatatePlatform.Value-180),0,0,1);				// Styring
			SetMaterial(MaterialType.BRONSJE);

			//Tegner platform som pendel skal kjre p
			Gl.glPushMatrix();
			Gl.glTranslatef(0,0,-20);
			DrawDisk();
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslatef(0,0,-20);
			Gl.glTranslatef((trackBarReference.Value - 50f)/50f*200f,0,0);				// Posisjon
			DrawMarker();
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslatef(0,0,2f);
			Gl.glTranslatef(ED.X1_K*200,0,0);				// Setter posisjonen til pendel langs x-aksen
			Gl.glRotatef((ED.X3_K/(2f*3.14f))*360f,0,1,0);	// Setter Vinkel til pendel om y-aksen
			DrawPendel();
			Gl.glPopMatrix();
		}
		public void DrawSimulatorTexst()
		{
			Gl.glEnable(Gl.GL_COLOR_MATERIAL);                                  // Enable Coloring Of Material
			string text1 = "Data fra simulator";
			string text2 = "Data til simulator";
			string text3 = "Akselerasjon    = " + String.Format("{0}{1:F2}",ED.Acceleration<0?'-':'+',Math.Abs(ED.Acceleration)) + " m/s^2";
			string text4 = "Posisjon         = " + String.Format("{0}{1:F2}",ED.X1_K<0?'-':'+',Math.Abs(ED.X1_K)) + " m"; 
			string text5 = "Hastighet       = " + String.Format("{0}{1:F2}",ED.X2_K<0?'-':'+',Math.Abs(ED.X2_K)) + " m/s"; 
			string text6 = "Vinkel           = " + String.Format("{0}{1:F2}",ED.X3_K<0?'-':'+',Math.Abs((ED.X3_K/(2f*3.14f))*360f)) + " deg"; 
			string text7 = "Vinkelhastighet = " + String.Format("{0}{1:F2}",ED.X4_K<0?'-':'+',Math.Abs((ED.X4_K/(2f*3.14f))*360f)) + " deg/s"; 
			string text8 = ED.RegOn?"Regulator er P ":"Regulator er AV ";
			string text9 =   "nsket Posisjon = " + String.Format("{0}{1:F2}",ED.PositionRef<0?'-':'+',Math.Abs(ED.PositionRef/37.4340599f)) +" m ";

			Gl.glColor3f(0.8f,0,0);

			Gl.glPushMatrix();
			SetMaterial(MaterialType.VIT);
			Gl.glTranslatef(-150,-250,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*1.1f/160f*Math.PI)*30f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.2f/160f*Math.PI)*20f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.3f/160f*Math.PI)*10f,0,0,1);
			Gl.glScalef(80,100,50);
			Gl.glTranslatef(-GetTextLenght(text1) / 2, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text1);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glColor3f(0.4f,0,0);

			Gl.glPushMatrix();
			Gl.glTranslatef(-50,-250,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*1.1f/160f*Math.PI)*3f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.2f/160f*Math.PI)*2f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.3f/160f*Math.PI)*1f,0,0,1);
			Gl.glScalef(45,45,10);
			Gl.glTranslatef(-5.5f, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text3);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslatef(0,-250,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*0.7f/160f*Math.PI)*3f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*0.9f/160f*Math.PI)*2f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.2f/160f*Math.PI)*1f,0,0,1);
			Gl.glScalef(45,45,10);
			Gl.glTranslatef(-5.5f, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text4);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslatef(50,-250,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*0.9f/160f*Math.PI)*3f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.1f/160f*Math.PI)*2f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*0.6f/160f*Math.PI)*1f,0,0,1);
			Gl.glScalef(45,45,10);
			Gl.glTranslatef(-5.5f, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text5);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslatef(100,-250,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*0.51f/160f*Math.PI)*3f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.5f/160f*Math.PI)*2f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.1f/160f*Math.PI)*1f,0,0,1);
			Gl.glScalef(45,45,10);
			Gl.glTranslatef(-5.5f, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text6);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslatef(150,-250,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*1.2f/160f*Math.PI)*3f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*0.9f/160f*Math.PI)*2f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*0.7f/160f*Math.PI)*1f,0,0,1);
			Gl.glScalef(45,45,10);
			Gl.glTranslatef(-5.5f, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text7);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glColor3f(0,0.8f,0);

			Gl.glPushMatrix();
			SetMaterial(MaterialType.VIT);
			Gl.glTranslatef(-150,300,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*1.3f/160f*Math.PI)*30f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.2f/160f*Math.PI)*20f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.1f/160f*Math.PI)*10f,0,0,1);
			Gl.glScalef(80,100,50);
			Gl.glTranslatef(-GetTextLenght(text2) / 2, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text2);										// Skriver tekst til skjerm
			Gl.glPopMatrix();

			Gl.glColor3f(0,0.4f,0);

			Gl.glPushMatrix();
			Gl.glTranslatef(-50,300,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*0.7f/160f*Math.PI)*3f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*1.1f/160f*Math.PI)*2f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*0.9f/160f*Math.PI)*1f,0,0,1);
			Gl.glScalef(45,45,10);
			Gl.glTranslatef(-5.5f, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text8);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glPushMatrix();
			Gl.glTranslatef(0,300,0);
			Gl.glRotatef(90,0,0,1);
			Gl.glRotatef((float)Math.Cos(Angle*1.1f/160f*Math.PI)*3f,1,0,0);
			Gl.glRotatef((float)Math.Cos(Angle*0.8f/160f*Math.PI)*2f,0,1,0);
			Gl.glRotatef((float)Math.Cos(Angle*0.5f/160f*Math.PI)*1f,0,0,1);
			Gl.glScalef(45,45,10);
			Gl.glTranslatef(-5.5f, 0, 0);   // sentrerer teksten p skjermen
			glPrint(text9);										// Skriver tekst til skjerm				
			Gl.glPopMatrix();

			Gl.glFrontFace  ( Gl.GL_CCW );
			Gl.glDisable(Gl.GL_COLOR_MATERIAL);                                  // Enable Coloring Of Material

		}
		
		/// <summary>
		/// Funksjon som returnerer lengden p teksten som sendes som parameter i pixel
		/// </summary>
		/// <param name="text"></param>
		/// <returns></returns>
		public float GetTextLenght(string text)
		{
			float length = 0;                                                   // Variabel som lagrer lengden til teksten
			char[] chars = text.ToCharArray();                                  

			for(int loop = 0; loop < text.Length; loop++)	// lkke for  finner tekst lengden 
			{                     
				length += gmf[chars[loop]].gmfCellIncX;                         // Lagger til lengden for vert tegn
			}
			return length;
		}

		/// <summary>
		/// Funksjon som tegner hele opengl senen, denne bestr av 4 viewports
		/// </summary>
		/// <returns></returns>
		private bool DrawGLScene() 
		{
			Gl.glClear(Gl.GL_COLOR_BUFFER_BIT);			// Sletter skjerm og dybde buffer

			for (int ViewPort=0; ViewPort<4; ViewPort++)
			{
				// Setter opp Vindu 1, Hovedvindu sett fra siden
				if (ViewPort == 0 && checkBoxView4.Checked) 
				{
					Gl.glViewport(0, 0, panel1.Width, panel1.Height-200);				// Setter Viewport
					Gl.glMatrixMode(Gl.GL_PROJECTION);                                  // Velger projeksjons modus
					Gl.glLoadIdentity();                                                // Resetter projeksjons matrisen
					Glu.gluPerspective(45, (panel1.Width) / (double) (panel1.Height-200), 0.1, 2000);	// beregner forholdet til vinduet
					Gl.glMatrixMode(Gl.GL_MODELVIEW);									// Velger Modelview matrisen
					Gl.glLoadIdentity();												// Reset  Modelview matrisen
					Gl.glClear(Gl.GL_DEPTH_BUFFER_BIT);									// Sletter dybde buffer, skjermbuffer kan ikke slettes her
					Glu.gluLookAt(trackBarZoom.Value ,0,100,0,0,50,0,0,1);				// Setter opp kamera posisjon
					// Tegner hele senen med pendelen		
					DrawPendulumScene();
				}
				// Setter opp Vindu 2, lite vindu sett ovenfra
				else if (ViewPort == 1 && checkBoxView3.Checked)
				{
					Gl.glViewport(panel1.Width-200, panel1.Height-200, 200, 200);       // Setter Viewport
					Gl.glMatrixMode(Gl.GL_PROJECTION);                                  // Velger projeksjons modus
					Gl.glLoadIdentity();                                                // Resetter projeksjons matrisen
					Glu.gluPerspective(45, (200) / (double) (200), 0.1, 2000);          // beregner forholdet til vinduet
					Gl.glMatrixMode(Gl.GL_MODELVIEW);									// Velger Modelview matrisen
					Gl.glLoadIdentity();												// Reset  Modelview matrisen
					Gl.glClear(Gl.GL_DEPTH_BUFFER_BIT);									// Sletter dybde buffer, skjermbuffer kan ikke slettes her
					Glu.gluLookAt(0,0,620,0,0,0,-1,0,0);								// Setter opp kamera posisjon
					// Tegner hele senen med pendelen				
					DrawPendulumScene();
				}
				// Setter opp Vindu 3, lite vindu med flgekamera
				else if (ViewPort == 2 && checkBoxView2.Checked)
				{
					Gl.glViewport(panel1.Width-400, panel1.Height-200, 200, 200);       // Setter Viewport
					Gl.glMatrixMode(Gl.GL_PROJECTION);                                  // Velger projeksjons modus
					Gl.glLoadIdentity();                                                // Resetter projeksjons matrisen
					Glu.gluPerspective(45, (200) / (double) (200), 0.1, 2000);          // beregner forholdet til vinduet
					Gl.glMatrixMode(Gl.GL_MODELVIEW);									// Velger Modelview matrisen
					Gl.glLoadIdentity();												// Reset  Modelview matrisen
					Gl.glClear(Gl.GL_DEPTH_BUFFER_BIT);									// Sletter dybde buffer, skjermbuffer kan ikke slettes her

					Gl.glRotatef(90,1,0,0);												// Setter opp kamera posisjon
					Gl.glRotatef(-180,0,1,0);											// Setter opp kamera posisjon
					Gl.glRotatef(-10,0,0,5);											// Setter opp kamera posisjon
					Gl.glTranslatef(-ED.X1_K*200+50,-180,-50);							// Setter opp kamera posisjon
					Gl.glRotatef(-(trackBarRoatatePlatform.Value-180),0,0,1);			// Setter opp kamera posisjon
					// Tegner hele senen med pendelen				
					DrawPendulumScene();
				}
     			// Setter opp Vindu 4, vindu med opengl tekst som beskriver simulator verdier 
				else if (ViewPort == 3  && checkBoxView1.Checked)
				{
					Gl.glViewport(0, panel1.Height-200, panel1.Width-400, 200);								// Setter Viewport
					Gl.glMatrixMode(Gl.GL_PROJECTION);                                  // Velger projeksjons modus
					Gl.glLoadIdentity();                                                // Resetter projeksjons matrisen
					Glu.gluPerspective(45, (panel1.Width-400) / (double) (200), 0.1, 2000);	// beregner forholdet til vinduet
					Gl.glMatrixMode(Gl.GL_MODELVIEW);									// Velger Modelview matrisen
					Gl.glLoadIdentity();												// Reset  Modelview matrisen
					Gl.glClear(Gl.GL_DEPTH_BUFFER_BIT);									// Sletter dybde buffer, skjermbuffer kan ikke slettes her
					Glu.gluLookAt(0,0,620,0,0,0,-1,0,0);								// Setter opp kamera posisjon

					DrawSimulatorTexst();
				}
			}
			return true;
		}

		#endregion

		#region Bitmap LoadBMP(string fileName)
		/// <summary>
		/// Funksjon som lager en bitmap fra spesifisert filnavn
		/// </summary>
		/// <param name="fileName"></param>
		/// <returns></returns>
		private static Bitmap LoadBMP(string fileName) 
		{
			if(fileName == null || fileName == string.Empty)			// Har vi et filnavn
			{					
				return null;											                                        
			}

			string fileName1 = string.Format("Data{0}{1}",              // Leter etter filnavn
				Path.DirectorySeparatorChar, fileName);
			string fileName2 = string.Format("{0}{1}{0}{1}Data{1}{2}",         
				"..", Path.DirectorySeparatorChar, fileName);

			// Filen m ligge i en av flgende kataloger
			if(!File.Exists(fileName) && !File.Exists(fileName1) && !File.Exists(fileName2)) 
			{
				return null;                                                    
			}

			// Leter opp filen
			if(File.Exists(fileName)) 
			{                                        
				return new Bitmap(fileName);    // Laster inn bitmap
			}
			else if(File.Exists(fileName1)) 
			{                                   
				return new Bitmap(fileName1);   // Laster inn bitmap
			}
			else if(File.Exists(fileName2)) 
			{                                   
				return new Bitmap(fileName2);   // Laster inn bitmap
			}

			return null;                        // Returnerer null vis ingen fil ble funnet
		}
		#endregion Bitmap LoadBMP(string fileName)

		#region bool LoadGLTextures()
		/// <summary>
		/// Funksjon som laster inn en tekstur
		/// </summary>
		/// <returns></returns>
		private static bool LoadGLTextures() 
		{
			bool status = false;                                                // Status 
			Bitmap[] textureImage = new Bitmap[1];                              // Lagringsplass for teksturen

			textureImage[0] = LoadBMP("plattform.bmp");							// Henter bitmap

			if(textureImage[0] != null) 
			{
				status = true;                                                  

				Gl.glGenTextures(1, out texture[0]);                            // Lager teksturen

				textureImage[0].RotateFlip(RotateFlipType.RotateNoneFlipY);     
				// Rektangel for lsing av bitmap i minne
				Rectangle rectangle = new Rectangle(0, 0, textureImage[0].Width, textureImage[0].Height);
				// Henter bitmap data
				BitmapData bitmapData = textureImage[0].LockBits(rectangle, ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb);

				// Tekstur generering
				Gl.glBindTexture(Gl.GL_TEXTURE_2D, texture[0]);
				Gl.glTexImage2D(Gl.GL_TEXTURE_2D, 0, Gl.GL_RGB8, textureImage[0].Width, textureImage[0].Height, 0, Gl.GL_BGR, Gl.GL_UNSIGNED_BYTE, bitmapData.Scan0);
				Gl.glTexParameteri(Gl.GL_TEXTURE_2D, Gl.GL_TEXTURE_MIN_FILTER, Gl.GL_LINEAR);
				Gl.glTexParameteri(Gl.GL_TEXTURE_2D, Gl.GL_TEXTURE_MAG_FILTER, Gl.GL_LINEAR);

				if(textureImage[0] != null) 
				{                                   
					textureImage[0].UnlockBits(bitmapData);                     // Figir data
					textureImage[0].Dispose();                                  // Dispose 
				}
			}

			return status;                                                      
		}
		#endregion bool LoadGLTextures()
		
		#region Materialer
		enum MaterialType 
		{
			BRONSJE,
			ROD,
			VIT,
			GRONN,
			GRAA,
			GLASS,
			KOBBER
		}

		void SetMaterial(MaterialType type)
		{

			switch (type)
			{
				case MaterialType.BRONSJE:
					float []mat_ambient={0.529412f,0.223529f,0.027451f};
					float []mat_diffuse={0.480392f,0.568627f,0.113725f };
					float []mat_specular={0.592157f, 0.641176f, 0.507843f };
					Gl.glMaterialfv (Gl.GL_FRONT,Gl.GL_AMBIENT,mat_ambient);
					Gl.glMaterialfv (Gl.GL_FRONT,Gl.GL_DIFFUSE,mat_diffuse);
					Gl.glMaterialfv (Gl.GL_FRONT,Gl.GL_SPECULAR,mat_specular);
					Gl.glMaterialf (Gl.GL_FRONT,Gl.GL_SHININESS,27.89743616f);
					break;
				case MaterialType.ROD:
				{
					float []amb={0.69f,0.0f,0.0f,1.0f};
					float []diff={0.3f,0.15f,0.11f,1.0f};
					float []spec={0.39f,0.49f,0.25f,1.0f};
					float shine=5f;
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_AMBIENT,amb);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_DIFFUSE,diff);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_SPECULAR,spec);
					Gl.glMaterialf(Gl.GL_FRONT_AND_BACK,Gl.GL_SHININESS,shine);
					break;
				}
				case MaterialType.VIT:
				{
					float []amb={0.69f,0.67f,0.45f,1.0f};
					float []diff={0.3f,0.15f,0.11f,1.0f};
					float []spec={0.39f,0.49f,0.25f,1.0f};
					float shine=5f;
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_AMBIENT,amb);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_DIFFUSE,diff);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_SPECULAR,spec);
					Gl.glMaterialf(Gl.GL_FRONT_AND_BACK,Gl.GL_SHININESS,shine);
					break;
				}
				case MaterialType.GRONN:
				{
					float []amb={0.0f,0.36f,0.0f,1.0f};
					float []diff={0.11f,0.26f,0.19f,1.0f};
					float []spec={0.25f,0.49f,0.25f,1.0f};
					float shine=7f;
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_AMBIENT,amb);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_DIFFUSE,diff);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_SPECULAR,spec);
					Gl.glMaterialf(Gl.GL_FRONT_AND_BACK,Gl.GL_SHININESS,shine);
					break;
				}
				case MaterialType.GRAA:
				{
					float []amb={0.32f,0.32f,0.33f,1.0f};
					float []diff={0.3f,0.29f,0.29f,1.0f};
					float []spec={0.27f,0.27f,0.25f,1.0f};
					float shine=7f;
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_AMBIENT,amb);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_DIFFUSE,diff);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_SPECULAR,spec);
					Gl.glMaterialf(Gl.GL_FRONT_AND_BACK,Gl.GL_SHININESS,shine);
					break;
				}
				case MaterialType.GLASS:
				{
					float []amb={0.0f,0.0f,0.0f,1.0f};
					float []diff={0.55f,0.55f,0.55f,1.0f};
					float []spec={0.7f,0.7f,0.7f,1.0f};
					float shine=32f;
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_AMBIENT,amb);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_DIFFUSE,diff);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_SPECULAR,spec);
					Gl.glMaterialf(Gl.GL_FRONT_AND_BACK,Gl.GL_SHININESS,shine);
					break;
				}
				case MaterialType.KOBBER:
				{
					float []amb={0.19125f,0.0735f,0.0225f,1.0f};
					float []diff={0.7038f,0.27048f,0.0828f,1.0f};
					float []spec={0.256777f,0.137622f,0.086014f,1.0f};
					float shine=12.8f;
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_AMBIENT,amb);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_DIFFUSE,diff);
					Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK,Gl.GL_SPECULAR,spec);
					Gl.glMaterialf(Gl.GL_FRONT_AND_BACK,Gl.GL_SHININESS,shine);
					break;
				}
				default:
					break;
			}
			
		}

		#endregion

		#region Overrides
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.RefreshTimer = new System.Windows.Forms.Timer(this.components);
			this.panel1 = new System.Windows.Forms.Panel();
			this.label8 = new System.Windows.Forms.Label();
			this.label7 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.panel5 = new System.Windows.Forms.Panel();
			this.panel3 = new System.Windows.Forms.Panel();
			this.panel2 = new System.Windows.Forms.Panel();
			this.trackBarRoatatePlatform = new System.Windows.Forms.TrackBar();
			this.label3 = new System.Windows.Forms.Label();
			this.trackBarZoom = new System.Windows.Forms.TrackBar();
			this.label2 = new System.Windows.Forms.Label();
			this.CpuUsagetimer = new System.Windows.Forms.Timer(this.components);
			this.progressBarCpuUsage = new System.Windows.Forms.ProgressBar();
			this.label4 = new System.Windows.Forms.Label();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.checkBoxView3 = new System.Windows.Forms.CheckBox();
			this.checkBoxView4 = new System.Windows.Forms.CheckBox();
			this.checkBoxView2 = new System.Windows.Forms.CheckBox();
			this.checkBoxView1 = new System.Windows.Forms.CheckBox();
			this.checkBoxTexture = new System.Windows.Forms.CheckBox();
			this.checkBox2 = new System.Windows.Forms.CheckBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.checkBoxNoiseOn = new System.Windows.Forms.CheckBox();
			this.checkBoxRegulationOn = new System.Windows.Forms.CheckBox();
			this.label1 = new System.Windows.Forms.Label();
			this.trackBarReference = new System.Windows.Forms.TrackBar();
			this.panel1.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.trackBarRoatatePlatform)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.trackBarZoom)).BeginInit();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.trackBarReference)).BeginInit();
			this.SuspendLayout();
			// 
			// RefreshTimer
			// 
			this.RefreshTimer.Interval = 25;
			this.RefreshTimer.Tick += new System.EventHandler(this.RefreshTimer_Tick);
			// 
			// panel1
			// 
			this.panel1.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panel1.Controls.Add(this.label8);
			this.panel1.Controls.Add(this.label7);
			this.panel1.Controls.Add(this.label6);
			this.panel1.Controls.Add(this.label5);
			this.panel1.Controls.Add(this.panel5);
			this.panel1.Controls.Add(this.panel3);
			this.panel1.Controls.Add(this.panel2);
			this.panel1.Location = new System.Drawing.Point(6, 8);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(840, 696);
			this.panel1.TabIndex = 0;
			// 
			// label8
			// 
			this.label8.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label8.Location = new System.Drawing.Point(440, 0);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(10, 16);
			this.label8.TabIndex = 7;
			this.label8.Text = "2";
			// 
			// label7
			// 
			this.label7.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label7.Location = new System.Drawing.Point(640, 0);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(10, 16);
			this.label7.TabIndex = 6;
			this.label7.Text = "3";
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(0, 200);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(10, 16);
			this.label6.TabIndex = 5;
			this.label6.Text = "4";
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(0, 0);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(10, 16);
			this.label5.TabIndex = 4;
			this.label5.Text = "1";
			// 
			// panel5
			// 
			this.panel5.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.panel5.Location = new System.Drawing.Point(440, 0);
			this.panel5.Name = "panel5";
			this.panel5.Size = new System.Drawing.Size(2, 200);
			this.panel5.TabIndex = 3;
			// 
			// panel3
			// 
			this.panel3.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.panel3.Location = new System.Drawing.Point(639, 0);
			this.panel3.Name = "panel3";
			this.panel3.Size = new System.Drawing.Size(2, 200);
			this.panel3.TabIndex = 1;
			// 
			// panel2
			// 
			this.panel2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panel2.Location = new System.Drawing.Point(0, 199);
			this.panel2.Name = "panel2";
			this.panel2.Size = new System.Drawing.Size(848, 2);
			this.panel2.TabIndex = 0;
			// 
			// trackBarRoatatePlatform
			// 
			this.trackBarRoatatePlatform.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.trackBarRoatatePlatform.Location = new System.Drawing.Point(856, 504);
			this.trackBarRoatatePlatform.Maximum = 360;
			this.trackBarRoatatePlatform.Name = "trackBarRoatatePlatform";
			this.trackBarRoatatePlatform.Size = new System.Drawing.Size(200, 45);
			this.trackBarRoatatePlatform.TabIndex = 3;
			this.trackBarRoatatePlatform.TickFrequency = 10;
			this.trackBarRoatatePlatform.Value = 60;
			// 
			// label3
			// 
			this.label3.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.label3.Location = new System.Drawing.Point(864, 480);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(168, 23);
			this.label3.TabIndex = 8;
			this.label3.Text = "Rotere plattform";
			// 
			// trackBarZoom
			// 
			this.trackBarZoom.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.trackBarZoom.Location = new System.Drawing.Point(856, 592);
			this.trackBarZoom.Maximum = 1000;
			this.trackBarZoom.Minimum = 3;
			this.trackBarZoom.Name = "trackBarZoom";
			this.trackBarZoom.Size = new System.Drawing.Size(200, 45);
			this.trackBarZoom.TabIndex = 22;
			this.trackBarZoom.TickFrequency = 40;
			this.trackBarZoom.Value = 500;
			// 
			// label2
			// 
			this.label2.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.label2.Location = new System.Drawing.Point(864, 568);
			this.label2.Name = "label2";
			this.label2.TabIndex = 23;
			this.label2.Text = "Zoom";
			// 
			// CpuUsagetimer
			// 
			this.CpuUsagetimer.Enabled = true;
			this.CpuUsagetimer.Interval = 1000;
			this.CpuUsagetimer.Tick += new System.EventHandler(this.CpuUsagetimer_Tick);
			// 
			// progressBarCpuUsage
			// 
			this.progressBarCpuUsage.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.progressBarCpuUsage.Location = new System.Drawing.Point(856, 680);
			this.progressBarCpuUsage.Name = "progressBarCpuUsage";
			this.progressBarCpuUsage.Size = new System.Drawing.Size(200, 23);
			this.progressBarCpuUsage.Step = 1;
			this.progressBarCpuUsage.TabIndex = 25;
			// 
			// label4
			// 
			this.label4.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.label4.Location = new System.Drawing.Point(856, 656);
			this.label4.Name = "label4";
			this.label4.TabIndex = 26;
			this.label4.Text = "Totalt CPU bruk";
			// 
			// groupBox1
			// 
			this.groupBox1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.groupBox1.Controls.Add(this.checkBoxView3);
			this.groupBox1.Controls.Add(this.checkBoxView4);
			this.groupBox1.Controls.Add(this.checkBoxView2);
			this.groupBox1.Controls.Add(this.checkBoxView1);
			this.groupBox1.Controls.Add(this.checkBoxTexture);
			this.groupBox1.Controls.Add(this.checkBox2);
			this.groupBox1.Location = new System.Drawing.Point(856, 208);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(200, 240);
			this.groupBox1.TabIndex = 33;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Egenskaper P/Av";
			// 
			// checkBoxView3
			// 
			this.checkBoxView3.Checked = true;
			this.checkBoxView3.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxView3.Location = new System.Drawing.Point(24, 168);
			this.checkBoxView3.Name = "checkBoxView3";
			this.checkBoxView3.TabIndex = 40;
			this.checkBoxView3.Text = "Vis vindu 3";
			// 
			// checkBoxView4
			// 
			this.checkBoxView4.Checked = true;
			this.checkBoxView4.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxView4.Location = new System.Drawing.Point(24, 200);
			this.checkBoxView4.Name = "checkBoxView4";
			this.checkBoxView4.TabIndex = 39;
			this.checkBoxView4.Text = "Vis vindu 4";
			// 
			// checkBoxView2
			// 
			this.checkBoxView2.Checked = true;
			this.checkBoxView2.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxView2.Location = new System.Drawing.Point(24, 128);
			this.checkBoxView2.Name = "checkBoxView2";
			this.checkBoxView2.TabIndex = 38;
			this.checkBoxView2.Text = "Vis vindu 2";
			// 
			// checkBoxView1
			// 
			this.checkBoxView1.Checked = true;
			this.checkBoxView1.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxView1.Location = new System.Drawing.Point(24, 96);
			this.checkBoxView1.Name = "checkBoxView1";
			this.checkBoxView1.TabIndex = 37;
			this.checkBoxView1.Text = "Vis vindu 1";
			// 
			// checkBoxTexture
			// 
			this.checkBoxTexture.Checked = true;
			this.checkBoxTexture.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxTexture.Location = new System.Drawing.Point(24, 24);
			this.checkBoxTexture.Name = "checkBoxTexture";
			this.checkBoxTexture.TabIndex = 35;
			this.checkBoxTexture.Text = "Tekstur p";
			// 
			// checkBox2
			// 
			this.checkBox2.Checked = true;
			this.checkBox2.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBox2.Location = new System.Drawing.Point(24, 60);
			this.checkBox2.Name = "checkBox2";
			this.checkBox2.TabIndex = 34;
			this.checkBox2.Text = "Lys p";
			this.checkBox2.CheckedChanged += new System.EventHandler(this.LightOnOff_CheckedChanged);
			// 
			// groupBox2
			// 
			this.groupBox2.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.groupBox2.BackColor = System.Drawing.SystemColors.Control;
			this.groupBox2.Controls.Add(this.checkBoxNoiseOn);
			this.groupBox2.Controls.Add(this.checkBoxRegulationOn);
			this.groupBox2.Controls.Add(this.label1);
			this.groupBox2.Controls.Add(this.trackBarReference);
			this.groupBox2.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.groupBox2.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.groupBox2.Location = new System.Drawing.Point(856, 16);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(200, 168);
			this.groupBox2.TabIndex = 35;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Data til simulator";
			// 
			// checkBoxNoiseOn
			// 
			this.checkBoxNoiseOn.Checked = true;
			this.checkBoxNoiseOn.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxNoiseOn.Location = new System.Drawing.Point(8, 136);
			this.checkBoxNoiseOn.Name = "checkBoxNoiseOn";
			this.checkBoxNoiseOn.Size = new System.Drawing.Size(88, 24);
			this.checkBoxNoiseOn.TabIndex = 38;
			this.checkBoxNoiseOn.Text = "Sty p";
			this.checkBoxNoiseOn.CheckedChanged += new System.EventHandler(this.checkBoxNoiseOn_CheckedChanged);
			// 
			// checkBoxRegulationOn
			// 
			this.checkBoxRegulationOn.Checked = true;
			this.checkBoxRegulationOn.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxRegulationOn.Location = new System.Drawing.Point(8, 104);
			this.checkBoxRegulationOn.Name = "checkBoxRegulationOn";
			this.checkBoxRegulationOn.Size = new System.Drawing.Size(128, 24);
			this.checkBoxRegulationOn.TabIndex = 37;
			this.checkBoxRegulationOn.Text = "Regulator P";
			this.checkBoxRegulationOn.CheckedChanged += new System.EventHandler(this.checkBoxRegulationOn_CheckedChanged);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 32);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(112, 23);
			this.label1.TabIndex = 36;
			this.label1.Text = "nsket posisjon";
			// 
			// trackBarReference
			// 
			this.trackBarReference.Location = new System.Drawing.Point(8, 56);
			this.trackBarReference.Maximum = 100;
			this.trackBarReference.Name = "trackBarReference";
			this.trackBarReference.Size = new System.Drawing.Size(184, 45);
			this.trackBarReference.TabIndex = 35;
			this.trackBarReference.TickFrequency = 5;
			this.trackBarReference.Value = 50;
			this.trackBarReference.ValueChanged += new System.EventHandler(this.trackBarReference_Scroll);
			// 
			// Pendulum
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(1064, 715);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.progressBarCpuUsage);
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.trackBarZoom);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.trackBarRoatatePlatform);
			this.Controls.Add(this.label3);
			this.Name = "Pendulum";
			this.Text = "Grafisk Databehandling, Vren 2005, Rune Wilhelm Dragsnes";
			this.Load += new System.EventHandler(this.Pendulum_Load);
			this.panel1.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.trackBarRoatatePlatform)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.trackBarZoom)).EndInit();
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.trackBarReference)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		#region Startpunkt
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Pendulum());
		}
		#endregion

		#region Event Handlers
		/// <summary>
		/// Denne timeren str for tegning til DC. Dette er gjort i en timer for  f en
		/// konstant oppdateringfrekvens.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void RefreshTimer_Tick(object sender, System.EventArgs e)
		{
			Angle += 4f;

			if(!((this != null) && !DrawGLScene()))
			{                                                      
				Gdi.SwapBuffers(hDC);                  // Swap Buffers (Systemet benytter dobbel buffer for  ung flikker)
			}
		}

		/// <summary>
		/// Kalles nr formens strrelse endres
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form_Resize(object sender, EventArgs e) 
		{
			ReSizeGLScene(panel1.Width, panel1.Height);                             // Justerer strrelsen p vinduet
		}

		/// <summary>
		/// Eventhandler som kalles nr checkboksen for lys p/av endres
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void LightOnOff_CheckedChanged(object sender, System.EventArgs e)
		{
			if (checkBox2.Checked)
			{
				Gl.glEnable(Gl.GL_LIGHT0);
			}
			else
			{
				Gl.glDisable(Gl.GL_LIGHT0);
			}
		}

		/// <summary>
		/// Evenhandler som slr regulatoren p/av
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void checkBoxRegulationOn_CheckedChanged(object sender, System.EventArgs e)
		{
			if (ED.RegOn)
			{
				ED.RegOn = false;
				ED.X3_K = ED.X3_K +0.00001f; 
			}
			else
			{
				ED.RegOn = true;
			}

		}

		/// <summary>
		/// Eventhandler som setter nsket posisjon som regulatoren skal regulere seg mot
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void trackBarReference_Scroll(object sender, System.EventArgs e)
		{
			float tmp = (float)(trackBarReference.Value - 50f)/50f;
			ED.PositionRef = tmp * 37.4340599f;
		}

		/// <summary>
		/// Eventhandler som benyttes til  beregne CPU bruk
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void CpuUsagetimer_Tick(object sender, System.EventArgs e)
		{
			progressBarCpuUsage.Value = (int)(getCurrentCpuUsage()+0.5f);
		}

		/// <summary>
		/// Eventhandler som benyttes til  sette sty til regulatoren
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void checkBoxNoiseOn_CheckedChanged(object sender, System.EventArgs e)
		{
			ED.NoiseOn = checkBoxNoiseOn.Checked;
		} 
		#endregion

		#region Diverse funksjoner

		/// <summary>
		/// Returnerer nverende cpu bruk
		/// </summary>
		/// <returns></returns>
		public float getCurrentCpuUsage()
		{ 
			return cpuCounter.NextValue(); 
		}
		#endregion

		#region Tekst 

		/// <summary>
		/// Bygger opp font listen
		/// </summary>
		private static void BuildFont() 
		{
			IntPtr font;                                                        // Windows Font ID
			fontbase = Gl.glGenLists(256);                                      // Lagringsplass for 256 tegn

			font = Gdi.CreateFont(                                              // Lager fonten
				-12,                                                            // Hyde for font
				0,                                                              // Bredde
				0,                                                              
				0,                                                              // Vinkel
				Gdi.FW_BOLD,                                                    // Hyde
				false,                                                          // Italic
				false,                                                          // Underline
				false,                                                          // Strikeout
				Gdi.ANSI_CHARSET,                                               // Character Set Identifier
				Gdi.OUT_TT_PRECIS,                                              // Output Precision
				Gdi.CLIP_DEFAULT_PRECIS,                                        // Clipping Precision
				Gdi.ANTIALIASED_QUALITY,                                        // Output Quality
				Gdi.FF_DONTCARE | Gdi.DEFAULT_PITCH,                            // Family And Pitch
				"Comic Sans MS");                                               // Font navn

			Gdi.SelectObject(hDC, font);                                        // Velger font
			Wgl.wglUseFontOutlines(
				hDC,                                                            // Velger DC
				0,                                                              // frste tegn
				255,                                                            // Number Of Display Lists To Build
				fontbase,                                                       
				0,                                                              // Deviation From The True Outlines
				0.6f,                                                           // Font hyde
				Wgl.WGL_FONT_POLYGONS,                                          // Bruk polygoner
				gmf);                                                           // Buffer 
		}

		/// <summary>
		/// Sletter font listen
		/// </summary>
		private static void KillFont() 
		{
			Gl.glDeleteLists(fontbase, 256);                                    // Delete All 256 Characters
		}

		/// <summary>
		/// Skriver ut tekst fra en display list
		/// </summary>
		/// <param name="text"></param>
		private static void glPrint(string text) 
		{
			Gl.glEnable(Gl.GL_NORMALIZE);

			Gl.glCullFace(Gl.GL_BACK);
			Gl.glEnable(Gl.GL_CULL_FACE);
			Gl.glPushMatrix();
			if(text == null || text.Length == 0) 
			{                              
				return;                                                         // returnere vis ingen tekst
			}

			Gl.glPushAttrib(Gl.GL_LIST_BIT);                                    // Pushes The Display List Bits
			Gl.glListBase(fontbase);											

			byte [] textbytes = new byte[text.Length];
			for (int i = 0; i < text.Length; i++) 
				textbytes[i] = (byte) text[i];
			Gl.glCallLists(text.Length, Gl.GL_UNSIGNED_BYTE, textbytes);		// Tegner tekst fra Display List
			Gl.glPopAttrib();												    
			Gl.glPopMatrix();
			Gl.glDisable(Gl.GL_CULL_FACE); // slr av culling
		}
		#endregion 

		private void Pendulum_Load(object sender, System.EventArgs e)
		{
		
		}

	}
}
