using System;
using System.Threading;
using System.Timers;

namespace PendulumEngine
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	
	public class Engine
	{
		private SharedEngineData ED;
		private System.Timers.Timer aTimer = new System.Timers.Timer();
		private System.Random rnd = new Random(100);

		public Engine(SharedEngineData Data)
		{
			ED = Data;
		}

		public void StartEngine()
		{
			aTimer.Elapsed += new ElapsedEventHandler(OnTimer);
			aTimer.Interval = 32;
			aTimer.Enabled = true;	
		}

		private void StateRegulation()
		{
			// legger til sty i regulatoren for  tilnerme virkeligheten
			ED.Position = ED.PositionRef - ED.X1_K;
			if (ED.NoiseOn)
			{
				ED.Position +=	(0.5f - (float)rnd.NextDouble())*10.0f; 
			}
			ED.Position = ED.Position + ED.PositionOld*0.032f;
			ED.PositionOld = ED.Position;
			ED.Position = ED.Position * -0.7499f;
			ED.Acceleration=ED.Position+( 28.2251f*ED.X1_K + 17.7442f*ED.X2_K + 91.4380f*ED.X3_K + 17.0714f*ED.X4_K);
		}

		public void OnTimer(Object source, ElapsedEventArgs e)
		{
			if (ED.RegOn)
			{
				StateRegulation();
				ED.X1_Kp1 =	ED.X1_K + 0.0328f*ED.X2_K - 0.0039f*ED.X3_K +		       0f + 0.0004f*ED.Acceleration;
				ED.X2_Kp1 =      0f +	  	  ED.X2_K - 0.2403f*ED.X3_K - 0.0039f*ED.X4_K + 0.0233f*ED.Acceleration;
				ED.X3_Kp1 =      0f +		       0f + 1.0213f*ED.X3_K + 0.0330f*ED.X4_K - 0.0009f*ED.Acceleration;
				ED.X4_Kp1 =      0f +		       0f + 1.3016f*ED.X3_K + 1.0213f*ED.X4_K - 0.0539f*ED.Acceleration;
			}
			else
			{			
								ED.X2_Kp1 = 0;
				ED.X3_Kp1 =      0f +		       0f + 1.0213f*ED.X3_K + 0.0330f*ED.X4_K - 0.0009f*ED.Acceleration;
				ED.X4_Kp1 =      0f +		       0f + 1.3016f*ED.X3_K + 1.0213f*ED.X4_K - 0.0539f*ED.Acceleration;
				if (ED.X3_Kp1 > 0.6f)
				{
					ED.X3_Kp1  = 0.6f;
					ED.X4_Kp1  = 0;
				}
				if (ED.X3_Kp1 < -0.6f)
				{
					ED.X3_Kp1  = -0.6f;
					ED.X4_Kp1  = 0;
				}			
			}

			ED.X1_K = ED.X1_Kp1;
			ED.X2_K = ED.X2_Kp1;
			ED.X3_K = ED.X3_Kp1;
			ED.X4_K = ED.X4_Kp1;
		}
	
	}


	public class SharedEngineData
	{
		private float x1_k;
		private float x2_k;
		private float x3_k;
		private float x4_k;
		private float x1_kp1;
		private float x2_kp1;
		private float x3_kp1;
		private float x4_kp1;
		private float position;
		private float positionOld;
		private float positionRef;
		private float acceleration;
		private bool  regOn;
		private bool noiseOn;

		public SharedEngineData()
		{
			x1_k = 0f;
			x2_k = 0f;
			x3_k = 0f;
			x4_k = 0f;
			x1_kp1 = 0f;
			x2_kp1 = 0f;
			x3_kp1 = 0f;
			x4_kp1 = 0f;
			position = 0f;
			positionOld = 0f;
			positionRef = 0f;
			acceleration = 0f;
			RegOn = true;
			NoiseOn = true;
		}
		private void EnterLock()
		{
			Monitor.Enter(this);
		}
		private void LeaveLock()
		{
			Monitor.Pulse(this);
			Monitor.Exit(this);			
		}


		public bool RegOn
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return regOn;
			}
			set
			{
				EnterLock();
				regOn = value;
				LeaveLock();
			}
		}
		public bool NoiseOn
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return noiseOn;
			}
			set
			{
				EnterLock();
				noiseOn = value;
				LeaveLock();
			}
		}
		
		public float X1_K
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x1_k;
			}
			set
			{
				EnterLock();
				x1_k = value;
				LeaveLock();
			}
		}
		public float X2_K
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x2_k;
			}
			set
			{
				EnterLock();
				x2_k = value;
				LeaveLock();
			}
		}
		public float X3_K
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x3_k;
			}
			set
			{
				EnterLock();
				x3_k = value;
				LeaveLock();
			}
		}
		public float X4_K
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x4_k;
			}
			set
			{
				EnterLock();
				x4_k = value;
				LeaveLock();
			}
		}
		public float X1_Kp1
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x1_kp1;
			}
			set
			{
				EnterLock();
				x1_kp1 = value;
				LeaveLock();
			}
		}
		public float X2_Kp1
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x2_kp1;
			}
			set
			{
				EnterLock();
				x2_kp1 = value;
				LeaveLock();
			}
		}
		public float X3_Kp1
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x3_kp1;
			}
			set
			{
				EnterLock();
				x3_kp1 = value;
				LeaveLock();
			}
		}
		public float X4_Kp1
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return x4_kp1;
			}
			set
			{
				EnterLock();
				x4_kp1 = value;
				LeaveLock();
			}
		}
		public float Position
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return position;
			}
			set
			{
				EnterLock();
				position = value;
				LeaveLock();
			}
		}
		public float PositionOld
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return positionOld;
			}
			set
			{
				EnterLock();
				positionOld = value;
				LeaveLock();
			}
		}
		public float PositionRef
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return positionRef;
			}
			set
			{
				EnterLock();
				positionRef = value;
				LeaveLock();
			}
		}
		public float Acceleration
		{
			get 
			{
				EnterLock();
				LeaveLock();
				return acceleration;
			}
			set
			{
				EnterLock();
				acceleration = value;
				LeaveLock();
			}
		}
	}

}
